/*
 *
 * Copyright 2002 Sun Microsystems, Inc. All rights reserved. 
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 * 
 */
import javax.xml.registry.*; 
import javax.xml.registry.infomodel.*; 
import java.net.*;
import java.security.*;
import java.util.*;

/**
 * The JAXRDelete class consists of a main method, a
 * makeConnection method, an executeQuery method, an 
 * executeRemove method, and a private helper method. It finds 
 * and deletes the organization that the JAXRPublish program 
 * created.  Specify the string key ID value returned by the 
 * JAXRPublish program.
 * 
 * To run this program, use the command 
 * 
 *     ant -Dkey-string=<value> run-delete
 */
public class JAXRDelete {
    Connection connection = null;
    RegistryService rs = null;

    public JAXRDelete() {}

    public static void main(String[] args) {
        String queryURL = 
            "http://www-3.ibm.com/services/uddi/v2beta/inquiryapi";
            //"http://uddi.rte.microsoft.com:/inquire";
            // For Registry Server, replace <YourHost> with fully
            //  qualified host name or localhost
            //"http://<YourHost>:8080/registry-server/RegistryServerServlet";
        String publishURL = 
            "https://www-3.ibm.com/services/uddi/v2beta/protect/publishapi";
            //"https://uddi.rte.microsoft.com/publish";
            // For Registry Server, replace <YourHost> with fully
            //  qualified host name or localhost
            //"http://<YourHost>:8080/registry-server/RegistryServerServlet";
        javax.xml.registry.infomodel.Key key = null;
        // Edit to provide your own username and password
        String username = "";
        String password = "";
        // If you changed the business name in JAXRPublish.java, edit
        //  to specify the beginning of your business name
        String busNameString = "The Coffee";

        if (args.length < 1) {
            System.out.println("Usage: ant " +
                "-Dkey-string=<value> run-delete");
            System.exit(1);
        }

        String keyString = new String(args[0]);
        System.out.println("Key string is " + keyString);

        JAXRDelete jd = new JAXRDelete();

        jd.makeConnection(queryURL, publishURL);

        key = jd.executeQuery(keyString, busNameString);

        if (key != null) {
            jd.executeRemove(key, username, password);
        } else {
            System.out.println("Not found, nothing to remove");
        }
    }
    
    /**
     * Establishes a connection to a registry.
     *
     * @param queryUrl	the URL of the query registry
     * @param publishUrl	the URL of the publish registry
     */
    public void makeConnection(String queryUrl, 
        String publishUrl) {

        /*
         * Edit to provide your own proxy information
         *  if you are going beyond your firewall.
         * Host format: "host.subdomain.domain.com".
         * Port is usually 8080.
         * Leave strings empty to use Registry Server.
         */
        String httpProxyHost = "";
        String httpProxyPort = "";
        String httpsProxyHost = "";
        String httpsProxyPort = "";

        /*
         * Define connection configuration properties. 
         * To delete, you need both the query URL and the 
         * publish URL.
         */
        Properties props = new Properties();
        props.setProperty("javax.xml.registry.queryManagerURL",
            queryUrl);
        props.setProperty("javax.xml.registry.lifeCycleManagerURL", 
            publishUrl);
        props.setProperty("javax.xml.registry.factoryClass", 
            "com.sun.xml.registry.uddi.ConnectionFactoryImpl");
        props.setProperty("javax.xml.registry.http.proxyHost", 
            httpProxyHost);
        props.setProperty("javax.xml.registry.http.proxyPort", 
            httpProxyPort);
        props.setProperty("javax.xml.registry.https.proxyHost", 
            httpsProxyHost);
        props.setProperty("javax.xml.registry.https.proxyPort", 
            httpsProxyPort);

        try {
            // Create the connection, passing it the 
            // configuration properties
            ConnectionFactory factory = 
                ConnectionFactory.newInstance();
            factory.setProperties(props);
            connection = factory.createConnection();
            System.out.println("Created connection to registry");
        } catch (Exception e) {
            e.printStackTrace();
            if (connection != null) {
                try {
                    connection.close();
                } catch (JAXRException je) {}
            }
        }
    }

    /**
     * Searches for the organization created by the JAXRPublish
     * program, verifying it by checking that the key strings
     * match.
     *
     * @param keyStr	the key of the published organization
     *
     * @return	the key of the organization found
     */
    public javax.xml.registry.infomodel.Key executeQuery(String keyStr,
        String busName) {

        BusinessQueryManager bqm = null;
        javax.xml.registry.infomodel.Key orgKey = null;

        try {
            rs = connection.getRegistryService();
            bqm = rs.getBusinessQueryManager();
            System.out.println("Got registry service and " +
                "query manager");

            // Define find qualifiers and name patterns
            Collection findQualifiers = new ArrayList();
            findQualifiers.add(FindQualifier.SORT_BY_NAME_DESC);
            Collection namePatterns = new ArrayList();
            namePatterns.add(busName);

            // Find using the name
            BulkResponse response = 
                bqm.findOrganizations(findQualifiers, 
                    namePatterns, null, null, null, null);
            Collection orgs = response.getCollection();

            // Find the organization submitted by JAXRPublish
            Iterator orgIter = orgs.iterator();
            boolean foundKey = false;
            String id = null;
            while (!foundKey && orgIter.hasNext()) {
                Organization org = 
                    (Organization) orgIter.next();
                System.out.println("Found " + getName(org));
                orgKey = org.getKey();
                id = orgKey.getId();
                foundKey = id.equals(keyStr);
            }
            if (foundKey) {
                System.out.println("Found organization with key " + id);
            } else {
                System.out.println("Organization not found");
                orgKey = null;
            }
        } catch (Exception e) {
            e.printStackTrace();
            if (connection != null) {
                try {
                    connection.close();
                } catch (JAXRException je) {
                    System.err.println("Connection close failed");
                }
            }
        }
        return orgKey;
    }

    /**
     * Removes the organization with the specified key value.
     *
     * @param key	the Key of the organization
     */
    public void executeRemove(javax.xml.registry.infomodel.Key key,
        String username, String password) {

        BusinessLifeCycleManager blcm = null;
    
        try {
            blcm = rs.getBusinessLifeCycleManager();

            // Get authorization from the registry
            PasswordAuthentication passwdAuth =
                new PasswordAuthentication(username, 
                    password.toCharArray());

            Set creds = new HashSet();
            creds.add(passwdAuth);
            connection.setCredentials(creds);
            System.out.println("Established security credentials");

            String id = key.getId();
            System.out.println("Deleting organization with id " + id);
            Collection keys = new ArrayList();
            keys.add(key);
            BulkResponse response = blcm.deleteOrganizations(keys);
            Collection exceptions = response.getException();
            if (exceptions == null) {
                System.out.println("Organization deleted");
                Collection retKeys = response.getCollection();
                Iterator keyIter = retKeys.iterator();
                javax.xml.registry.infomodel.Key orgKey = null;
                if (keyIter.hasNext()) {
                    orgKey = 
                        (javax.xml.registry.infomodel.Key) keyIter.next();
                    id = orgKey.getId();
                    System.out.println("Organization key was " + id);
                }
            } else {
                Iterator excIter = exceptions.iterator();
                Exception exception = null;
                while (excIter.hasNext()) {
                    exception = (Exception) excIter.next();
                    System.err.println("Exception on delete: " + 
                    exception.toString());
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally  {
            // At end, close connection to registry
            if (connection != null) {
                try {
                    connection.close();
                } catch (JAXRException je) {
                    System.err.println("Connection close failed");
                }
            }
        }
    }

    /**
     * Returns the name value for a registry object.  Returns
     * an empty string if the value is null.
     *
     * @param ro	a RegistryObject
     * @return		the String value
     */
    private String getName(RegistryObject ro) throws JAXRException {
        try {
            return ro.getName().getValue();
        } catch (NullPointerException npe) {
            return "No Name";
        }
    }
}
